# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import sys

try:
    import codc as odc
except Exception:
    import pyodc as odc
    print("Using pure Python odc decoder.", file=sys.stderr)


class ODBReader:

    def __init__(self, path):
        self.path = path

    def to_pandas(self):
        return odc.read_odb(self.path, single=True)



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

from importlib import import_module
import os


def lookup(name):
    source = import_module('.%s' % (name.replace('-', '_'),), package=__name__)
    return source.source


def load(name, *args, **kwargs):
    return lookup(name)(*args, **kwargs)


def list_entries():
    here = os.path.realpath(os.path.dirname(__file__))
    result = []

    for n in os.listdir(here):
        if n.startswith('.'):
            continue

        if n.startswith('_'):
            continue

        if not n.endswith('.py'):
            continue

        result.append(n[:-3])

    return result


class DataSource:

    sphinxdoc = '''
        No help
    '''

    # def __iter__(self):
    #     raise NotImplementedError("%r: method __iter__() not implemented" % (self.__class__.__name__,))

    # def __len__(self):
    #     raise NotImplementedError("%r: method __len__() not implemented" % (self.__class__.__name__,))

    # def __getitem__(self, n):
    #     raise NotImplementedError("%r: method __getitem__() not implemented" % (self.__class__.__name__,))

    # def to_xarray(self, *args, **kwargs):
    #     raise NotImplementedError("%r: method to_xarray() not implemented" % (self.__class__.__name__,))

    # def to_pandas(self, *args, **kwargs):
    #     raise NotImplementedError("%r: method to_pandas() not implemented" % (self.__class__.__name__,))

    # def to_numpy(self, *args, **kwargs):
    #     raise NotImplementedError("%r: method to_numpy() not implemented" % (self.__class__.__name__,))

    # def to_metview(self, *args, **kwargs):
    #     raise NotImplementedError("%r: method to_metview() not implemented" % (self.__class__.__name__,))



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import os
from . import DataSource
from climetlab.core.caching import temp_file
import requests
from tqdm import tqdm
import zipfile
import xarray as xr

class ZipUrl(DataSource):

    def __init__(self, url):
        self.path = temp_file("ZipUrl", url)

        if not os.path.exists(self.path + '.d'):
            if not os.path.exists(self.path + '.zip'):
                print("Downloading", url)
                r = requests.head(url)
                r.raise_for_status()
                size = int(r.headers['content-length'])

                r = requests.get(url, stream=True)
                r.raise_for_status()
                total = 0
                mode = 'wb'
                with tqdm(total=size,
                          unit_scale=True,
                          unit_divisor=1024,
                          unit='B',
                          disable=False,
                          leave=False,
                          ) as pbar:
                    pbar.update(total)
                    with open(self.path, mode) as f:
                        for chunk in r.iter_content(chunk_size=1024):
                            if chunk:
                                f.write(chunk)
                                total += len(chunk)
                                pbar.update(len(chunk))
                os.rename(self.path, self.path + '.zip')
            print("Unzipping...")
            if not os.path.exists(self.path + '.tmp'):
                os.mkdir(self.path + '.tmp')

            with zipfile.ZipFile(self.path + '.zip', "r") as zip_file:
                for file in tqdm(iterable=zip_file.namelist(),
                                 leave=False,
                                 total=len(zip_file.namelist())):
                    zip_file.extract(member=file, path=self.path + '.tmp')

            print("Done...")
            os.rename(self.path + '.tmp', self.path + '.d')
            os.unlink(self.path + '.zip')

        self._xarray = xr.open_mfdataset(self.path + '.d/*.nc', combine='by_coords')  # nested

    def to_xarray(self):
        return self._xarray


source = ZipUrl



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

# The code is copied from skinnywms, and we should combile later

from contextlib import closing
from itertools import product

import xarray as xr
import datetime
import numpy as np


def as_datetime(self, time):
    return datetime.datetime.strptime(str(time)[:19], "%Y-%m-%dT%H:%M:%S")


def as_level(self, level):
    n = float(level)
    if int(n) == n:
        return int(n)
    return n

class Slice:

    def __init__(self, name, value, index, is_dimension, is_info):
        self.name = name
        self.index = index
        self.value = value
        self.is_dimension = not is_info,
        self.is_info = is_info

    def __repr__(self):
        return "[%s:%s=%s]" % (self.name, self.index, self.value)


class TimeSlice(Slice):
    pass


class Coordinate:

    def __init__(self, variable, info):
        self.variable = variable
        # We only support 1D coordinate for now
        # assert len(variable.dims) == 1
        self.is_info = info
        self.is_dimension = not info

        if variable.values.ndim == 0:
            self.values = [self.convert(variable.values)]
        else:
            self.values = [self.convert(t) for t in variable.values][:10]

    def make_slice(self, value):
        return self.slice_class(self.variable.name, value, self.values.index(value), self.is_dimension, self.is_info)

    def __repr__(self):
        return "%s[name=%s,values=%s]" % (self.__class__.__name__, self.variable.name, len(self.values))


class TimeCoordinate(Coordinate):
    slice_class = TimeSlice
    is_dimension = True
    convert = as_datetime


class LevelCoordinate(Coordinate):
    # This class is just in case we want to specialise
    # 'level', othewise, it is the same as OtherCoordinate
    slice_class = Slice
    is_dimension = False
    convert = as_level


class OtherCoordinate(Coordinate):
    slice_class = Slice
    is_dimension = False
    convert = as_level


class NetCDFField:

    def __init__(self, path, ds, variable, slices):

        dims = ds[variable].dims

        latitude = ds[variable][dims[-2]]
        longitude = ds[variable][dims[-1]]

        self.north = np.amax(latitude.data)
        self.south = np.amin(latitude.data)
        self.east = np.amax(longitude.data)
        self.west = np.amin(longitude.data)

        self.path = path
        self.variable = variable
        self.slices = slices

        self.name = self.variable

        self.title = getattr(ds[self.variable], 'long_name',
                             getattr(ds[self.variable], 'standard_name',
                                     self.variable))

        self.time = None

        for s in self.slices:

            if isinstance(s, TimeSlice):
                self.time = s.value

            if s.is_info:
                self.title += ' (' + s.name + '=' + str(s.value) + ')'

    def plot_map(self, driver):

        dimensions = ["%s:%s" % (s.name, s.index) for s in self.slices]

        if dimensions:
            params = dict(netcdf_filename=self.path,
                          netcdf_value_variable=self.variable,
                          netcdf_dimension_setting=dimensions,
                          netcdf_dimension_setting_method='index')
        else:
            params = dict(netcdf_filename=self.path,
                          netcdf_value_variable=self.variable)

        # params['netcdf_field_automatic_scaling'] = 'off'

        driver.bounding_box(north=self.north,
                            south=self.south,
                            west=self.west,
                            east=self.east)

        driver.plot_netcdf(params)

    def __repr__(self):
        return "NetCDFField[%r,%r]" % (self.variable, self.slices)

class NetCDFReader:

    def __init__(self, path):
        self.path = path
        self.fields = None

    def _scan(self):
        if self.fields is None:
            self.fields = self.get_fields()

    def __repr__(self):
        return "NetCDFReader(%s)" % (self.path,)

    def __iter__(self):
        self._scan()
        return iter(self.fields)

    def __len__(self):
        self._scan()
        return len(self.fields)

    def __getitem__(self, n):
        self._scan()
        return self.fields[n]

    def get_fields(self):
        with closing(xr.open_mfdataset(self.path, combine='by_coords')) as ds:  # or nested
            return self._get_fields(ds)

    def _get_fields(self, ds):
        # Select only geographical variables

        fields = []

        skip = set()

        for name in ds.data_vars:
            v = ds[name]
            skip.update([c for c in getattr(v, 'coordinates', '').split(' ')])
            skip.update([c for c in getattr(v, 'bounds', '').split(' ')])

        for name in ds.data_vars:

            if name in skip:
                continue

            v = ds[name]

            coordinates = []

            # self.log.info('Scanning file: %s var=%s coords=%s', self.path, name, v.coords)

            info = [value for value in v.coords if value not in v.dims]

            for coord in v.coords:
                c = ds[coord]

                # self.log.info("COORD %s %s %s %s", coord, type(coord), hasattr(c, 'calendar'), c)

                standard_name = getattr(c, 'standard_name', None)
                axis = getattr(c, 'axis', None)
                long_name = getattr(c, 'long_name', None)

                use = False

                if standard_name in ('longitude', 'projection_x_coordinate') or (long_name == 'longitude'):
                    has_lon = True
                    use = True

                if standard_name in ('latitude', 'projection_y_coordinate') or (long_name == 'latitude'):
                    has_lat = True
                    use = True

                # Of course, not every one sets the standard_name
                if standard_name in ('time', 'forecast_reference_time') or axis == 'T':
                    coordinates.append(TimeCoordinate(c, coord in info))
                    use = True

                # TODO: Support other level types
                if standard_name in ('air_pressure', 'model_level_number', 'altitude'):  # or axis == 'Z':
                    coordinates.append(LevelCoordinate(c, coord in info))
                    use = True

                if axis in ('X', 'Y'):
                    use = True

                if not use:
                    coordinates.append(OtherCoordinate(c, coord in info))

            if not (has_lat and has_lon):
                # self.log.info("NetCDFReader: skip %s (Not a 2 field)", name)
                continue

            for values in product(*[c.values for c in coordinates]):

                slices = []
                for value, coordinate in zip(values, coordinates):
                    slices.append(coordinate.make_slice(value))

                fields.append(NetCDFField(self.path, ds, name, slices))

        if not fields:
            raise Exception("NetCDFReader no 2D fields found in %s", self.path)

        return fields

    def to_xarray(self):
        import xarray as xr
        return xr.open_dataset(self.path)



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

# This is needed when running Sphinx on ReadTheDoc
try:
    from .drivers.magics import Driver

except Exception:
    from .drivers.missing import Driver

from importlib import import_module

try:
    from IPython.display import display
except Exception:
    def display(x):
        return x

CURRENT_DRIVER = Driver()

HELPERS = {
    'xarray.core.dataset.Dataset': 'xarray',
    'xarray.core.dataarray.DataArray': 'xarray',
    'numpy.ndarray': 'ndarray',
    'pandas.core.frame.DataFrame': 'pandas',
}


def plot_map(data, *args, **kwargs):
    # This is a standalone plot, so we reset the driver
    CURRENT_DRIVER = Driver(*args, **kwargs)
    if getattr(data, 'plot_map', None) is None:

        fullname = '.'.join([data.__class__.__module__, data.__class__.__qualname__])

        name = HELPERS.get(fullname)

        if name is not None:
            helper = import_module('.helpers.%s' % (name,), package=__name__)
            data = helper.helper(data, *args, **kwargs)
        else:
            raise ValueError("Cannot plot %s" % (fullname,))

    data.plot_map(CURRENT_DRIVER)

    return display(CURRENT_DRIVER.show(**kwargs))



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

from .import Dataset
from climetlab import load_source


class SampleBUFRData(Dataset):

    def __init__(self, url='http://download.ecmwf.int/test-data/metview/gallery/temp.bufr'):
        self.source = load_source("url", url)


dataset = SampleBUFRData



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

from .sources import load as load_source
from .datasets import load as load_dataset
from .plotting import plot_map


__version__ = '0.0.26'

import climetlab.source
import climetlab.dataset


__all__ = [load_source,
           load_dataset,
           plot_map,
           climetlab.dataset,
           climetlab.source]



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import logging

LOG = logging.getLogger(__name__)

try:
    from climetlab.grib_bindings import GribFile

    class Reader(GribFile):
        pass

    LOG.info("Using eccodes C bindings to decode GRIB data")

except AttributeError:  # eccodes not installed
    import pyeccodes

    class Reader(pyeccodes.Reader):
        def at_offset(self, offset):
            self.seek(offset)
            return next(self)

    LOG.info("Using pyeccodes to decode GRIB data")


class GribField:

    def __init__(self, handle, path):
        self.handle = handle
        self.path = path

    @property
    def values(self):
        return self.handle.get('values')

    def plot_map(self, driver):
        driver.bounding_box(north=self.handle.get('latitudeOfFirstGridPointInDegrees'),
                            south=self.handle.get('latitudeOfLastGridPointInDegrees'),
                            west=self.handle.get('longitudeOfFirstGridPointInDegrees'),
                            east=self.handle.get('longitudeOfLastGridPointInDegrees'))
        driver.plot_grib(self.path, self.handle.get('offset'))

    def to_numpy(self):
        return self.values.reshape((self.handle.get("Nj"),
                                    self.handle.get("Ni")))

    @property
    def offset(self):
        return int(self.handle.get("offset"))

    def __repr__(self):
        return "GribField(%s,%s,%s,%s,%s,%s)" % (self.handle.get("shortName"),
                                                 self.handle.get("levelist"),
                                                 self.handle.get("date"),
                                                 self.handle.get("time"),
                                                 self.handle.get("step"),
                                                 self.handle.get("number"))

    def grid_definition(self):
        return dict(north=self.handle.get('latitudeOfFirstGridPointInDegrees'),
                    south=self.handle.get('latitudeOfLastGridPointInDegrees'),
                    west=self.handle.get('longitudeOfFirstGridPointInDegrees'),
                    east=self.handle.get('longitudeOfLastGridPointInDegrees'),
                    south_north_increment=self.handle.get('jDirectionIncrementInDegrees'),
                    west_east_increment=self.handle.get('iDirectionIncrementInDegrees'))

    def metadata(self):
        m = dict()
        for n in ('shortName', 'units'):
            p = self.handle.get(n)
            if p is not None:
                m[n] = str(p)

        return m


class GRIBIterator:

    def __init__(self, path):
        self.path = path
        self.reader = Reader(path)

    def __repr__(self):
        return "GRIBIterator(%s)" % (self.path,)

    def __next__(self):
        return GribField(next(self.reader), self.path)


class GRIBReader:

    def __init__(self, path):
        self.path = path
        self._fields = None
        self._reader = None

    def __repr__(self):
        return "GRIBReader(%s)" % (self.path,)

    def __iter__(self):
        return GRIBIterator(self.path)

    def _items(self):
        if self._fields is None:
            self._fields = []
            # print("Scan", self.path)
            for f in self:
                self._fields.append(f.offset)
        return self._fields

    def __getitem__(self, n):
        if self._reader is None:
            self._reader = Reader(self.path)
        return GribField(self._reader.at_offset(self._items()[n]), self.path)

    def __len__(self):
        return len(self._items())

    def to_xarray(self):
        import xarray as xr
        return xr.open_dataset(self.path, engine='cfgrib')



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import numpy as np
import xarray as xr

class XArrayPlotter:

    def __init__(self, data, *args, **kwargs):

        self.data = data
        for name, var in data.data_vars.items():
            self.name = name
            self.var = var
            self.extra_dims = var.dims[:-2]
            break

        lat, lon = self.var.dims[-2], self.var.dims[-1]

        data[lat].attrs['standard_name'] = 'latitude'
        data[lon].attrs['standard_name'] = 'longitude'

        self.north = np.amax(data[lat].data)
        self.south = np.amin(data[lat].data)
        self.east = np.amax(data[lon].data)
        self.west = np.amin(data[lon].data)

    def plot_map(self, driver):
        driver.bounding_box(north=self.north,
                            south=self.south,
                            west=self.west,
                            east=self.east)

        dimension_settings = dict()

        for d in self.extra_dims:
            dimension_settings[d] = self.data[d].data[0]

        driver.plot_xarray(self.data, self.name, dimension_settings)


def helper(data, *args, **kwargs):
    if isinstance(data, xr.Dataset):
        return XArrayPlotter(data, *args, **kwargs)
    else:
        return XArrayPlotter(data.to_dataset(), *args, **kwargs)



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import os
import tempfile

# Examples
# https://github.com/ecmwf/notebook-examples/tree/master/visualisation

from Magics import macro

try:
    from IPython.display import Image, SVG
except Exception:
    def Image(x):
        return x

    def SVG(x):
        return x

class Driver:

    def __init__(self, width=680, grid=False, **kwargs):
        self._projection = None
        self._data = None
        self._format = 'png'
        self._width_cm = 10
        self._height_cm = 10
        self._width = width
        self._page_ratio = 1.0
        self._contour = macro.mcont(contour_automatic_setting='ecmwf',
                                    legend='off',)

        self._grid = grid
        self._background = macro.mcoast(map_grid=self._grid,
                                        map_grid_colour='tan',
                                        map_label='off',
                                        map_boundaries='on',
                                        map_coastline_land_shade='on',
                                        map_coastline_land_shade_colour='cream',
                                        map_coastline_colour='tan')

        self._foreground = macro.mcoast(map_grid=self._grid,
                                        map_label='off')

        self._legend = None
        self._title = None

        self.kwargs = dict(**kwargs)

        self.bounding_box(90, -180, -90, 180)

    def bounding_box(self, north, west, south, east):
        self._projection = macro.mmap(subpage_upper_right_longitude=float(east),
                                      subpage_upper_right_latitude=float(north),
                                      subpage_lower_left_latitude=float(south),
                                      subpage_lower_left_longitude=float(west),
                                      subpage_map_projection='cylindrical')
        self._page_ratio = (north - south) / (east - west)

    def plot_grib(self, path, offset):
        self._data = macro.mgrib(grib_input_file_name=path,
                                 grib_file_address_mode='byte_offset',
                                 grib_field_position=int(offset))

    def plot_netcdf(self, params):
        self._data = macro.mnetcdf(**params)

    def plot_numpy(self, data, north, west, south_north_increment, west_east_increment, metadata):
        self._data = macro.minput(input_field=data,
                                  input_field_initial_latitude=float(north),
                                  input_field_latitude_step=-float(south_north_increment),
                                  input_field_initial_longitude=float(west),
                                  input_field_longitude_step=float(west_east_increment),
                                  input_metadata=metadata,)

    def plot_xarray(self, ds, variable, dimension_settings={}):
        self._data = macro.mxarray(xarray_dataset=ds,
                                   xarray_variable_name=variable,
                                   xarray_dimension_settings=dimension_settings)

    def plot_geopoints(self, path):
        self._data = macro.mgeo(geo_input_file_name=path)
        self._contour = macro.msymb(legend="off",
                                    symbol_type="marker",
                                    symbol_colour="red",
                                    symbol_height=0.08,
                                    # symbol_marker_index=20
                                    )

    def contouring(self, contouring):
        self._contour = macro.mcont(contouring)

    def plot_values(self, latitudes, longitudes, values, metadata={}):
        self._data = macro.minput(input_type='geographical',
                                  input_values=list(values),
                                  input_latitudes_list=list(latitudes),
                                  input_longitudes_list=list(longitudes),
                                  input_metadata=metadata,
                                  )

    def show(self,
             path=None,
             width=None,
             title=None,
             format=None,
             frame=False, **kwargs):

        if format:
            self._format = format

        if "projection" in self.kwargs:
            projection = self.kwargs["projection"]
            if isinstance(projection, str):
                self._projection = macro.mmap(subpage_map_projection=projection)
            else:
                self._projection = macro.mmap(**projection)

        tmp = False
        if path is None:
            tmp = True
            fd, path = tempfile.mkstemp('.' + self._format)
            os.close(fd)

        _title_height_cm = 0
        if title:
            _title_height_cm = 0.7
            if title is True:
                # Automatic title
                self._title = macro.mtext()
            else:
                self._title = macro.mtext(
                    text_lines=[title],
                    # text_justification='center',
                    # text_font_size=0.6,
                    # text_mode="positional",
                    # text_box_x_position=5.00,
                    # text_box_y_position=18.50,
                    # text_colour='charcoal'
                )

        base, fmt = os.path.splitext(path)
        output = macro.output(output_formats=[fmt[1:]],
                              output_name_first_page_number='off',
                              page_x_length=float(self._width_cm),
                              page_y_length=float(self._height_cm) * self._page_ratio,
                              super_page_x_length=float(self._width_cm),
                              super_page_y_length=float(self._height_cm) * self._page_ratio + _title_height_cm,
                              subpage_x_length=float(self._width_cm),
                              subpage_y_length=float(self._height_cm) * self._page_ratio,
                              subpage_x_position=0.,
                              subpage_y_position=0.,
                              output_width=self._width if width is None else width,
                              page_frame=frame,
                              page_id_line='off',
                              output_name=base)

        args = [x for x in (output,
                            self._projection,
                            self._background,
                            self._data,
                            self._contour,
                            self._foreground,
                            self._legend,
                            self._title) if x is not None]

        macro.plot(*args)

        if self._format == 'svg':
            Display = SVG
        else:
            Display = Image

        if tmp:
            img = Display(path, metadata=dict(width=width))
            os.unlink(path)
            return img

        return Display(path)



# (C) Copyright 2020 ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation nor
# does it submit to any jurisdiction.
#

import pandas as pd
import os

from . import Dataset
from climetlab import load_source


class MeteonetGroundStations(Dataset):
    """
    See https://github.com/meteofrance/meteonet
    """

    def __init__(self, domain='NW', date='20160101'):

        URL = "https://github.com/meteofrance/meteonet/raw/master/data_samples/ground_stations"
        url = "{url}/{domain}_{date}.csv".format(url=URL, domain=domain, date=date)
        self.path = load_source("url", url).path
        self._pandas = pd.read_csv(self.path, parse_dates=[4], infer_datetime_format=True)

    def to_pandas(self):
        return self._pandas

    def plot_map(self, driver):

        north, east = self._pandas[['lat', 'lon']].max()
        south, west = self._pandas[['lat', 'lon']].min()

        lats = self._pandas['lat'].to_numpy()
        lons = self._pandas['lon'].to_numpy()
        vals = self._pandas['t'].to_numpy()

        driver.bounding_box(north, west,
                            south, east)

        try:
            os.unlink(self.path + '.geo')
        except Exception:
            pass

        with open(self.path + '.geo', 'w') as f:
            print("#GEO", file=f)
            print("#FORMAT XYV", file=f)
            print("min: {}".format(min(vals)), file=f)
            print("max: {}".format(max(vals)), file=f)
            print("x/long  y/lat  value", file=f)
            print("#DATA", file=f)
            for lat, lon, v in zip(lats, lons, vals):
                print(lon, lat, v, file=f)

        driver.plot_geopoints(self.path + '.geo')
        # driver.plot_values(latitudes=lats,
        #                    longitudes=lons,
        #                    values=vals)

        # driver.contouring({})
        # driver.contouring(dict(contour_grid_value_plot=True,
        #                        contour=False,
        #                        contour_grid_value_plot_type='marker'))


dataset = MeteonetGroundStations
